@@ -15,7 +15,7 @@ from dataclasses_json import dataclass_json
 from ngwidgets.yamlable import YamlAble
 from slugify import slugify
 
-from dcm.svg import SVG, SVGConfig, SVGNodeConfig
+from dcm.svg import SVG, SVGNodeConfig
 
 
 @dataclass_json
@@ -26,6 +26,7 @@ class CompetenceElement:
 
     Attributes:
         name (str): The name of the competence element.
+        short_name(Optional[str]): the label to be displayed
         id (Optional[str]): An optional identifier for the competence element will be set to the name if id is None.
         url (Optional[str]): An optional URL for more information about the competence element.
         description (Optional[str]): An optional description of the competence element.
@@ -33,6 +34,7 @@ class CompetenceElement:
     """
 
     name: str
+    short_name: Optional[str] = None
     id: Optional[str] = None
     url: Optional[str] = None
     description: Optional[str] = None
@@ -41,7 +43,10 @@ class CompetenceElement:
     def __post_init__(self):
         # Set the id to the the slug of the name if id is None
         if self.id is None:
-            self.id = slugify(self.name)
+            # https://pypi.org/project/python-slugify/
+            self.id = slugify(self.name, lowercase=False, regex_pattern=r"[^\w\s\-]")
+        if self.short_name is None:
+            self.short_name = self.name[:10]
 
     def as_html(self) -> str:
         """
@@ -91,6 +96,7 @@ class CompetenceFacet(CompetenceElement):
     This class can include additional properties or methods specific to a competence facet.
     """
 
+
 @dataclass_json
 @dataclass
 class CompetenceArea(CompetenceElement):
@@ -100,8 +106,10 @@ class CompetenceArea(CompetenceElement):
     Attributes:
         facets (List[CompetenceFacet]): A list of CompetenceFacet objects representing individual facets of this area.
     """
+
     facets: List[CompetenceFacet] = field(default_factory=list)
 
+
 @dataclass_json
 @dataclass
 class CompetenceAspect(CompetenceElement):
@@ -146,28 +154,44 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
     """
 
     lookup_url: Optional[str] = None
+    total_levels: int = field(init=False)
     aspects: List[CompetenceAspect] = field(default_factory=list)
     levels: List[CompetenceLevel] = field(default_factory=list)
     element_names: Dict[str, str] = field(default_factory=dict)
+    total_elements: Dict[str, int] = field(default_factory=dict)
 
     def __post_init__(self):
         """
         initalize the path variables of my hierarchy
         """
         super().__post_init__()
+        self.total_elements = {"aspects": 0, "areas": 0, "facets": 0}
+        self.update_paths()
+
+    def update_paths(self):
+        """
+        update my paths
+        """
         self.path = self.id
+        self.total_levels = 1
         # Loop through each competence aspect and set their paths and parent references
         for aspect in self.aspects:
             aspect.competence_tree = self
             aspect.path = f"{self.id}/{aspect.id}"
+            self.total_elements["aspects"] = self.total_elements["aspects"] + 1
+            self.total_levels = 2
             for area in aspect.areas:
+                self.total_levels = 3
                 area.competence_tree = self
                 area.aspect = aspect
                 area.path = f"{self.id}/{aspect.id}/{area.id}"
+                self.total_elements["areas"] = self.total_elements["areas"] + 1
                 for facet in area.facets:
+                    self.total_levels = 4
                     facet.competence_tree = self
                     facet.area = area
                     facet.path = f"{self.id}/{aspect.id}/{area.id}/{facet.id}"
+                    self.total_elements["facets"] = self.total_elements["facets"] + 1
 
     @classmethod
     def required_keys(cls) -> Tuple:
@@ -209,7 +233,9 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
         if len(parts) > 1:
             aspect_id = parts[1]
             # Retrieve the aspect
-            aspect = next((aspect for aspect in self.aspects if aspect.id==aspect_id), None)
+            aspect = next(
+                (aspect for aspect in self.aspects if aspect.id == aspect_id), None
+            )
         if aspect:
             if len(parts) == 2:
                 return aspect
@@ -229,6 +255,33 @@ class CompetenceTree(CompetenceElement, YamlAble["CompetenceTree"]):
         handle_error(f"invalid path for lookup {path}")
         return None
 
+    @property
+    def total_valid_levels(self) -> int:
+        """
+        Calculate the total number of levels excluding
+        levels with a level of 0.
+
+        Returns:
+            int: The total number of valid levels.
+        """
+        level_count = len([level for level in self.levels if level.level != 0])
+        return level_count
+
+    def get_level_color(self, achievement_level: int) -> Optional[str]:
+        """
+        Retrieve the color associated with a specific achievement level.
+
+        Args:
+            achievement_level (int): The level of achievement to get the color for.
+
+        Returns:
+            Optional[str]: The color code associated with the given level, or None if not found.
+        """
+        for level in self.levels:
+            if level.level == achievement_level:
+                return level.color_code
+        return None
+
     def to_pretty_json(self):
         """
         Converts the CompetenceTree object to a pretty JSON string, handling null values.
@@ -407,12 +460,12 @@ class DynamicCompetenceMap:
     a visualization of a competence map
     """
 
-    def __init__(self, competence_tree: CompetenceTree):
+    def __init__(self, competence_tree: CompetenceTree, svg: SVG = None):
         """
         constructor
         """
         self.competence_tree = competence_tree
-        self.svg = None
+        self.svg = svg
 
     @property
     def main_id(self):
@@ -486,10 +539,10 @@ class DynamicCompetenceMap:
             ValueError: If an unsupported markup format is specified.
         """
         if markup == "json":
-            data=json.loads(text)
+            data = json.loads(text)
             return data
         elif markup == "yaml":
-            data=yaml.safe_load(text)
+            data = yaml.safe_load(text)
             return data
         else:
             raise ValueError(f"Unsupported markup format: {markup}")
@@ -533,7 +586,12 @@ class DynamicCompetenceMap:
 
     @classmethod
     def from_definition_string(
-        cls, name: str, definition_string: str, content_class, markup: str = "json"
+        cls,
+        name: str,
+        definition_string: str,
+        content_class,
+        markup: str = "json",
+        debug: bool = False,
     ) -> Any:
         """
         Load a DynamicCompetenceMap or Learner instance from a definition string (either JSON or YAML).
@@ -543,7 +601,7 @@ class DynamicCompetenceMap:
             definition_string (str): The string content of the definition.
             content_class (dataclass_json): The class which will be instantiated with the parsed data.
             markup (str): The markup format of the data. Defaults to 'json'. Supported values are 'json' and 'yaml'.
-
+            debug(bool): if True supply a JSON dump of the data in /tmp/{name}.json
         Returns:
             DynamicCompetenceMap: An instance of DynamicCompetenceMap loaded with the parsed data.
 
@@ -552,6 +610,11 @@ class DynamicCompetenceMap:
         """
         try:
             data = cls.parse_markup(definition_string, markup)
+            if debug:
+                # Save the parsed data to a JSON file in /tmp directory
+                debug_file_path = os.path.join("/tmp", f"{name}.json")
+                with open(debug_file_path, "w") as debug_file:
+                    json.dump(data, debug_file, indent=2, default=str)
             content = content_class.from_dict(data)
             if isinstance(content, CompetenceTree):
                 return DynamicCompetenceMap(content)
