@@ -1,10 +1,9 @@
 import html
+import math
+from dataclasses import dataclass, field
 from datetime import datetime
-from math import cos, radians, sin
 from typing import List, Optional, Tuple
 
-from pydantic.dataclasses import dataclass
-
 
 @dataclass
 class SVGConfig:
@@ -19,8 +18,8 @@ class SVGConfig:
         font_size (int): Font size in points for text elements.
         indent (str): Indentation string, default is two spaces.
         default_color (str): Default color code for SVG elements.
+        with_pop(bool): if True support popup javascript functionality
     """
-
     width: int = 600
     height: int = 600
     legend_height: int = 150
@@ -28,6 +27,7 @@ class SVGConfig:
     font_size: int = 12
     indent: str = "  "
     default_color: str = "#C0C0C0"
+    with_popup: bool=False
 
     @property
     def total_height(self) -> int:
@@ -39,39 +39,168 @@ class SVGConfig:
         """
         return self.height + self.legend_height
 
+    @property
+    def line_height(self) -> float:
+        # Calculate line height based on font size
+        # You can adjust this multiplier as needed
+        line_height = self.font_size * 1.2
+        return line_height
+
 
 @dataclass
-class SVGNodeConfig:
+class SVGNode:
     """
-    a single SVG Node configuration
+    a generic SVG Node
     """
 
-    x: float
-    y: float
+    indent_level: int = 1
+    id: Optional[str] = None
+    color: Optional[
+        str
+    ] = None  # Color of font or stroke use default color of config if None
+    fill: Optional[str] = "black"  # Fill color for the segment
+    title: Optional[str] = None  # Tooltip
+    comment: Optional[str] = None
+
+
+@dataclass
+class SVGNodeConfig(SVGNode):
+    """
+    a single SVG Node configuration
+    to display any element
+    """
+
+    x: float = 0.0
+    y: float = 0.0
     width: Optional[float] = None
     height: Optional[float] = None
-    fill: Optional[str] = "black"
-    indent_level: int = 1
     element_type: Optional[str] = None
-    id: Optional[str] = None
-    title: Optional[str] = None
     url: Optional[str] = None
     show_as_popup: bool = False  # Flag to indicate if the link should opened as a popup
-    comment: Optional[str] = None
     element_class: Optional[str] = "hoverable"
 
 
 @dataclass
-class DonutSegment:
+class Text:
     """
-    a donut segment
+    Class to handle text-related operations in SVG.
+
+    Attributes:
+        text (str): The text content.
+        config (SVGConfig): Configuration for SVG generation.
     """
 
-    inner_radius: float
-    outer_radius: float
+    text: str
+    config: SVGConfig
+    lines: List[str] = field(init=False)
+
+    def __post_init__(self):
+        """
+        Post-initialization processing to split the text into lines.
+        """
+        self.lines = self.text.strip().split("\n")
+
+    @property
+    def line_count(self) -> int:
+        return len(self.lines)
+
+    @property
+    def total_text_height(self) -> float:
+        """
+        Calculate the total height of the text based on the number of lines and the line height.
+
+        Returns:
+            float: The total height of the text in pixels.
+        """
+        height = self.config.line_height * len(self.lines)
+        return height
+
+
+@dataclass
+class Arc:
+    radius: float
+    start_x: float
+    start_y: float
+    end_x: float
+    end_y: float
+
+    # may be calculated by the get_arc function of DonutSegment
+    middle_x: Optional[float] = None
+    middle_y: Optional[float] = None
+
+
+@dataclass
+class DonutSegment(SVGNode):
+    """
+    A donut segment representing a
+    section of a donut chart.
+    """
+
+    cx: float = 0.0
+    cy: float = 0.0
+    inner_radius: float = 0.0
+    outer_radius: float = 0.0
     start_angle: Optional[float] = 0.0
     end_angle: Optional[float] = 360.0
 
+    @property
+    def large_arc_flag(self) -> str:
+        """
+        Determine if the arc should be drawn as a large-arc (values >= 180 degrees).
+
+        Returns:
+            str: "1" if the arc is a large arc, otherwise "0".
+        """
+        large_arc_flag = "1" if self.end_angle - self.start_angle >= 180 else "0"
+        return large_arc_flag
+
+    def relative_angle(self, angle_factor=0.5) -> float:
+        """Calculate a relative angle of the donut segment."""
+        relative_angle = (self.start_angle + self.end_angle) * angle_factor
+        return relative_angle
+
+    def radial_radius(self, radial_offset: float) -> float:
+        """
+        get the radial radius for the given radial offset
+        """
+        radial_radius = (
+            self.inner_radius + (self.outer_radius - self.inner_radius) * radial_offset
+        )
+        return radial_radius
+
+    def x_y(self, angle: float, radial_radius: float):
+        x = self.cx + radial_radius * math.cos(math.radians(angle))
+        y = self.cy + radial_radius * math.sin(math.radians(angle))
+        return x, y
+
+    def get_arc(self, radial_offset: float = 0.5) -> Arc:
+        """
+        Get the Arc for the givne radial offset
+
+        Args:
+            radial_offset(float): e.g. 0.0 - inner 1.0 outer 0.5 middle
+        Returns:
+            Arc: the arc at the given radial offset
+        """
+        # Calculate the adjusted radius within the bounds of inner and outer radii
+        radial_radius = self.radial_radius(radial_offset)
+
+        # Calculate the start and end points of the arc
+        start_x, start_y = self.x_y(self.start_angle, radial_radius)
+        end_x, end_y = self.x_y(self.end_angle, radial_radius)
+        middle_angle = self.relative_angle(0.5)
+        middle_x, middle_y = self.x_y(middle_angle, radial_radius)
+
+        return Arc(
+            radius=radial_radius,
+            start_x=start_x,
+            start_y=start_y,
+            middle_x=middle_x,
+            middle_y=middle_y,
+            end_x=end_x,
+            end_y=end_y,
+        )
+
 
 class SVG:
     """
@@ -94,6 +223,13 @@ class SVG:
         self.elements = []
         self.indent = self.config.indent
 
+    def get_indent(self, level) -> str:
+        """
+        get the indentation for the given level
+        """
+        indentation = f"{self.indent * level}"
+        return indentation
+
     def get_svg_style(self) -> str:
         """
         Define styles for SVG elements.
@@ -101,33 +237,40 @@ class SVG:
         Returns:
             str: String containing style definitions for SVG.
         """
-        return (
+        style = (
             f"{self.indent}<style>\n"
             f"{self.indent * 2}.hoverable {{ cursor: pointer; fill-opacity: 1; stroke: black; stroke-width: 0.5; }}\n"
             f"{self.indent * 2}.hoverable:hover {{ fill-opacity: 0.7; }}\n"
             f"{self.indent * 2}.selected {{ fill-opacity: 0.5; stroke: blue; stroke-width: 1.5;}}\n"
-            f"{self.indent * 2}.popup {{\n"
-            f"{self.indent * 3}border: 2px solid black;\n"
-            f"{self.indent * 3}border-radius: 15px;\n"
-            f"{self.indent * 3}overflow: auto;\n"  # changed to 'auto' to allow scrolling only if needed
-            f"{self.indent * 3}background: white;\n"
-            f"{self.indent * 3}box-sizing: border-box;\n"  # ensures padding and border are included
-            f"{self.indent * 3}padding: 10px;\n"  # optional padding inside the popup
-            f"{self.indent * 3}height: 100%;\n"  # adjusts height relative to foreignObject
-            f"{self.indent * 3}width: 100%;\n"  # adjusts width relative to foreignObject
-            f"{self.indent * 2}}}\n"
-            f"{self.indent * 2}.close-btn {{\n"  # style for the close button
-            f"{self.indent * 3}cursor: pointer;\n"
-            f"{self.indent * 3}position: absolute;\n"
-            f"{self.indent * 3}top: 0;\n"
-            f"{self.indent * 3}right: 0;\n"
-            f"{self.indent * 3}padding: 5px;\n"
-            f"{self.indent * 3}font-size: 20px;\n"
-            f"{self.indent * 3}user-select: none;\n"  # prevents text selection on click
-            f"{self.indent * 2}}}\n"
-            f"{self.indent}</style>\n"
+            f"{self.indent * 2}.noclick {{ pointer-events: none; }}\n"  # style for non-clickable text
         )
 
+        if self.config.with_popup:
+            style += (
+                f"{self.indent * 2}.popup {{\n"
+                f"{self.indent * 3}border: 2px solid black;\n"
+                f"{self.indent * 3}border-radius: 15px;\n"
+                f"{self.indent * 3}overflow: auto;\n"  # changed to 'auto' to allow scrolling only if needed
+                f"{self.indent * 3}background: white;\n"
+                f"{self.indent * 3}box-sizing: border-box;\n"  # ensures padding and border are included
+                f"{self.indent * 3}padding: 10px;\n"  # optional padding inside the popup
+                f"{self.indent * 3}height: 100%;\n"  # adjusts height relative to foreignObject
+                f"{self.indent * 3}width: 100%;\n"  # adjusts width relative to foreignObject
+                f"{self.indent * 2}}}\n"
+                f"{self.indent * 2}.close-btn {{\n"  # style for the close button
+                f"{self.indent * 3}cursor: pointer;\n"
+                f"{self.indent * 3}position: absolute;\n"
+                f"{self.indent * 3}top: 0;\n"
+                f"{self.indent * 3}right: 0;\n"
+                f"{self.indent * 3}padding: 5px;\n"
+                f"{self.indent * 3}font-size: 20px;\n"
+                f"{self.indent * 3}user-select: none;\n"  # prevents text selection on click
+                f"{self.indent * 2}}}\n"
+            )
+
+        style += f"{self.indent}</style>\n"
+        return style
+
     def get_text_width(self, text: str) -> int:
         """
         Estimate the width of a text string in the SVG based on the font size and font name.
@@ -142,16 +285,76 @@ class SVG:
         average_char_width = average_char_width_factor * self.config.font_size
         return int(average_char_width * len(text))
 
-    def add_element(self, element: str, level: int = 1, comment: str = None):
+    def get_text_rotation(self, rotation_angle: float) -> float:
+        """
+        Adjusts the rotation angle for SVG text elements to ensure that the text
+        is upright and readable in a circular chart. The text will be rotated
+        by 180 degrees if it is in the lower half of the chart (between 90 and 270 degrees).
+
+        Args:
+            rotation_angle (float): The initial rotation angle of the text element.
+
+        Returns:
+            float: The adjusted rotation angle for the text element.
+        """
+        # In the bottom half of the chart (90 to 270 degrees), the text
+        # would appear upside down, so we rotate it by 180 degrees.
+        if 90 <= rotation_angle < 270:
+            rotation_angle -= 180
+
+        # Return the adjusted angle. No adjustment is needed for the
+        # top half of the chart as the text is already upright.
+        return rotation_angle
+
+    def get_donut_path(
+        self,
+        segment: DonutSegment,
+        radial_offset: float = 0.5,
+        middle_arc: bool = False,
+    ) -> str:
+        """
+        Create an SVG path definition for an arc using the properties of a DonutSegment.
+
+        Args:
+            segment (DonutSegment): The segment for which to create the path.
+            radial_offset(float): 0 to 1 - useable in middle_arc mode
+            middle_arc(bool): if True get the middle arc
+
+        Returns:
+            str: SVG path definition string for the full donut segment or the middle_arc if middle_arc is set to true.
+        """
+        if middle_arc:
+            arc = segment.get_arc(radial_offset=radial_offset)
+
+            # Create the path for the middle arc
+            path_str = (
+                f"M {arc.start_x} {arc.start_y} "  # Move to start of middle arc
+                f"A {arc.radius} {arc.radius} 0 {segment.large_arc_flag} 1 {arc.end_x} {arc.end_y}"
+            )
+        else:
+            outer_arc = segment.get_arc(radial_offset=1)
+            inner_arc = segment.get_arc(radial_offset=0)
+            path_str = (
+                f"M {inner_arc.start_x} {inner_arc.start_y} "  # Move to start of inner arc
+                f"L {outer_arc.start_x} {outer_arc.start_y} "  # Line to start of outer arc
+                f"A {segment.outer_radius} {segment.outer_radius} 0 {segment.large_arc_flag} 1 {outer_arc.end_x} {outer_arc.end_y} "  # Outer arc
+                f"L {inner_arc.end_x} {inner_arc.end_y} "  # Line to end of inner arc
+                f"A {segment.inner_radius} {segment.inner_radius} 0 {segment.large_arc_flag} 0 {inner_arc.start_x} {inner_arc.start_y} "  # Inner arc (reverse)
+                "Z"
+            )
+
+        return path_str
+
+    def add_element(self, element: str, indent_level: int = 1, comment: str = None):
         """
         Add an SVG element to the elements list with proper indentation.
 
         Args:
             element (str): SVG element to be added.
-            level (int): Indentation level for the element.
+            indent_level (int): Indentation level for the element.
             comment(str): optional comment to add
         """
-        base_indent = f"{self.indent * level}"
+        base_indent = self.get_indent(indent_level)
         if comment:
             indented_comment = f"{base_indent}<!-- {comment} -->\n"
             self.elements.append(indented_comment)
@@ -170,7 +373,7 @@ class SVG:
 
         # If URL is provided, wrap the circle in an anchor tag to make it clickable
         if config.url:
-            circle_indent = self.indent * (config.indent_level + 1)
+            circle_indent = self.get_indent(config.indent_level + 1)
             circle_element = f"""<a xlink:href="{config.url}" target="_blank">
 {circle_indent}{circle_element}
 </a>"""
@@ -180,7 +383,7 @@ class SVG:
             circle_element,
             group_id=config.id,
             group_class=config.element_class,
-            level=config.indent_level,
+            indent_level=config.indent_level,
             comment=config.comment,
         )
 
@@ -205,8 +408,8 @@ class SVG:
             indent_level (int): Indentation level for the rectangle.
         """
         color = fill if fill else self.config.default_color
-        rect = f'{self.indent * 3}<rect x="{x}" y="{y}" width="{width}" height="{height}" fill="{color}" />\n'
-        self.add_element(rect)
+        rect = f'<rect x="{x}" y="{y}" width="{width}" height="{height}" fill="{color}" />\n'
+        self.add_element(rect, indent_level=indent_level)
 
     def add_legend_column(
         self,
@@ -241,6 +444,10 @@ class SVG:
         fill: str = "black",
         font_weight: str = "normal",
         text_anchor: str = "start",
+        transform: str = "",
+        centered: bool = False,
+        text_class: str = "noclick",
+        indent_level: int = 1,
     ) -> None:
         """
         Add text to the SVG.
@@ -248,20 +455,43 @@ class SVG:
         Args:
             x (int): X position of the text.
             y (int): Y position of the text.
-            text (str): Text content.
+            text(str): Text content.
             fill (str, optional): Fill color of the text. Defaults to "black".
             font_weight (str, optional): Font weight (normal, bold, etc.). Defaults to "normal".
             text_anchor (str, optional): Text alignment (start, middle, end). Defaults to "start".
+            indent_level(int): the indentation level to apply
+            centered (bool): If True, treat x and y as the center of the text. Default is False.
+            text_class(str): "noclick" by default so that elements below are clickable
+            transform (str, optional): Transformation for the text (e.g., rotation). Defaults to an empty string.
         """
-        escaped_text = html.escape(text)
+        text_obj = Text(text, self.config)
+        if centered:
+            # Adjust text_anchor to 'middle' when centered
+            text_anchor = "middle"
+
+            # y-offset adjustment to center the text vertically
+            y -= text_obj.total_text_height / 2
+            # adjust for the ascender / descender vertical font weighting
+            y -= self.config.font_size * 0.5
+        # Create a text element to hold the tspan elements
+        # Only include the transform attribute if it is provided
+        transform_attr = f'transform="{transform}" ' if transform else ""
+
         text_element = (
-            f'<text x="{x}" y="{y}" fill="{fill}" '
+            f'\n{self.get_indent(indent_level)}<text class="{text_class}" x="{x}" y="{y}" fill="{fill}" '
             f'font-family="{self.config.font}" '
             f'font-size="{self.config.font_size}" '
             f'font-weight="{font_weight}" '
-            f'text-anchor="{text_anchor}">'
-            f"{escaped_text}</text>\n"
+            f'text-anchor="{text_anchor}" '
+            f'dominant-baseline="middle" '
+            f"{transform_attr}>"
         )
+        # Add tspan elements for each line
+        for line in text_obj.lines:
+            escaped_line = html.escape(line)
+            text_element += f'\n{self.get_indent(indent_level+1)}<tspan x="{x}" dy="{self.config.line_height}">{escaped_line}</tspan>'
+
+        text_element += f"\n{self.get_indent(indent_level)}</text>\n"
         self.add_element(text_element)
 
     def add_group(
@@ -269,7 +499,7 @@ class SVG:
         content: str,
         group_id: str = None,
         group_class: str = None,
-        level: int = 1,
+        indent_level: int = 1,
         comment: str = None,
     ):
         """
@@ -279,7 +509,7 @@ class SVG:
             content (str): SVG content to be grouped.
             group_id (str, optional): ID for the group.
             group_class (str, optional): Class for the group.
-            level (int): Indentation level for the group.
+            indent_level (int): Indentation level for the group.
         """
         group_attrs = []
         if group_id:
@@ -288,84 +518,14 @@ class SVG:
             group_attrs.append(f'class="{group_class}"')
         attrs_str = " ".join(group_attrs)
         indented_content = "\n".join(
-            f"{self.indent * (level + 1)}{line}" for line in content.strip().split("\n")
-        )
-        group_str = f"{self.indent * level}<g {attrs_str}>\n{indented_content}\n{self.indent * level}</g>\n"
-        self.add_element(group_str, level=level, comment=comment)
-
-    def add_pie_segment(
-        self,
-        cx: int,
-        cy: int,
-        radius: int,
-        start_angle_deg: float,
-        end_angle_deg: float,
-        color: str,
-        segment_name: str,
-        segment_id: str = None,
-        segment_class: str = None,
-        segment_url: str = None,
-    ) -> None:
-        """
-        Add a pie segment to the SVG.
-
-        Args:
-            cx (int): X-coordinate of the center of the pie.
-            cy (int): Y-coordinate of the center of the pie.
-            radius (int): Radius of the pie.
-            start_angle_deg (float): Start angle of the segment in degrees.
-            end_angle_deg (float): End angle of the segment in degrees.
-            color (str): Fill color of the segment.
-            segment_name (str): Name of the segment, used for the tooltip.
-            segment_id (str, optional): ID for the segment group. Defaults to None.
-            segment_class (str, optional): Class for the segment group. Defaults to None.
-            segment_url (str, optional): URL linked to the segment. Defaults to None.
-
-        Returns:
-            None
-        """
-        if color is None:
-            color = self.config.default_color
-        # Convert angles from degrees to radians for calculations
-        start_angle_rad = radians(start_angle_deg)
-        end_angle_rad = radians(end_angle_deg)
-
-        # Calculate the start and end points
-        start_x = cx + radius * cos(start_angle_rad)
-        start_y = cy + radius * sin(start_angle_rad)
-        end_x = cx + radius * cos(end_angle_rad)
-        end_y = cy + radius * sin(end_angle_rad)
-
-        # Determine if the arc should be drawn as a large-arc (values >= 180 degrees)
-        large_arc_flag = "1" if end_angle_deg - start_angle_deg >= 180 else "0"
-
-        # Create the path for the pie segment without indentation
-        path_str = (
-            f"M {cx} {cy} "
-            f"L {start_x} {start_y} "
-            f"A {radius} {radius} 0 {large_arc_flag} 1 {end_x} {end_y} "
-            "Z"
-        )
-
-        # Assemble the path and title elements
-        path_element = f'<path d="{path_str}" fill="{color}" />\n'
-        escaped_title = html.escape(segment_name)  # Escape special characters
-
-        title_element = f"<title>{escaped_title}</title>"
-
-        # Combine path and title into one string without adding indentation here
-        group_content = f"{path_element}{title_element}"
-
-        # If an URL is provided, wrap the content within an anchor
-        if segment_url:
-            group_content = (
-                f'<a xlink:href="{segment_url}" target="_blank">\n{group_content}</a>\n'
-            )
-
-        # Use add_group to add the pie segment with proper indentation
-        self.add_group(
-            group_content, group_id=segment_id, group_class=segment_class, level=2
+            f"{self.get_indent(indent_level + 1)}{line}"
+            for line in content.strip().split("\n")
         )
+        group_str = f"""{self.get_indent(indent_level)}<g {attrs_str}>
+{indented_content}
+{self.get_indent(indent_level)}</g>
+"""
+        self.add_element(group_str, indent_level=indent_level, comment=comment)
 
     def add_donut_segment(
         self,
@@ -377,49 +537,23 @@ class SVG:
 
         Args:
             config (SVGNodeConfig): Configuration for the donut segment.
-            start_angle_deg (float): Start angle of the segment in degrees.
-            end_angle_deg (float): End angle of the segment in degrees.
+            segment(DonutSegment)
         """
-        cx, cy = config.x, config.y
         color = config.fill if config.fill else self.config.default_color
 
         if color is None:
             color = self.config.default_color
-        # Convert angles from degrees to radians for calculations
-        start_angle_rad = radians(segment.start_angle)
-        end_angle_rad = radians(segment.end_angle)
 
-        # Calculate the start and end points for the outer radius
-        start_x_outer = cx + segment.outer_radius * cos(start_angle_rad)
-        start_y_outer = cy + segment.outer_radius * sin(start_angle_rad)
-        end_x_outer = cx + segment.outer_radius * cos(end_angle_rad)
-        end_y_outer = cy + segment.outer_radius * sin(end_angle_rad)
-
-        # Calculate the start and end points for the inner radius
-        start_x_inner = cx + segment.inner_radius * cos(start_angle_rad)
-        start_y_inner = cy + segment.inner_radius * sin(start_angle_rad)
-        end_x_inner = cx + segment.inner_radius * cos(end_angle_rad)
-        end_y_inner = cy + segment.inner_radius * sin(end_angle_rad)
-
-        # Determine if the arc should be drawn as a large-arc (values >= 180 degrees)
-        large_arc_flag = "1" if segment.end_angle - segment.start_angle >= 180 else "0"
-
-        # Create the path for the pie segment without indentation
-        path_str = (
-            f"M {start_x_inner} {start_y_inner} "  # Move to start of inner arc
-            f"L {start_x_outer} {start_y_outer} "  # Line to start of outer arc
-            f"A {segment.outer_radius} {segment.outer_radius} 0 {large_arc_flag} 1 {end_x_outer} {end_y_outer} "  # Outer arc
-            f"L {end_x_inner} {end_y_inner} "  # Line to end of inner arc
-            f"A {segment.inner_radius} {segment.inner_radius} 0 {large_arc_flag} 0 {start_x_inner} {start_y_inner} "  # Inner arc (reverse)
-            "Z"
-        )
+        path_str = self.get_donut_path(segment)
 
         # Assemble the path and title elements
         path_element = f'<path d="{path_str}" fill="{color}" />\n'
-        escaped_title = html.escape(config.title)  # Escape special characters
-
-        title_element = f"<title>{escaped_title}</title>"
+        if config.title:
+            escaped_title = html.escape(config.title)  # Escape special characters
 
+            title_element = f"<title>{escaped_title}</title>"
+        else:
+            title_element = ""
         # Combine path and title into one string without adding indentation here
         group_content = f"{path_element}{title_element}"
 
@@ -439,10 +573,76 @@ class SVG:
             group_content,
             group_id=config.id,
             group_class=config.element_class,
-            level=2,
+            indent_level=2,
             comment=config.comment,
         )
 
+    def add_text_to_donut_segment(
+        self,
+        segment: DonutSegment,
+        text: str,
+        direction: str = "horizontal",
+        color: str = "white",
+        text_class: str = "noclick",
+        indent_level: int = 1,
+    ) -> None:
+        """
+        Add text to a donut segment with various direction options.
+
+        Args:
+            segment (DonutSegment): The donut segment to which text will be added.
+            text (str): The text content to be added.
+            direction (str): The direction in which the text should be drawn.
+                             Options are "horizontal", "angled", or "curved".
+            color (str): The color of the text. Default is "white".
+        """
+        if direction in ["horizontal", "angled"]:
+            # Calculate position for horizontal or angled text
+            # Use the get_arc method to find the position for horizontal or angled text
+            mid_arc = segment.get_arc(radial_offset=0.5)  # Get the mid-arc
+            text_x, text_y = mid_arc.middle_x, mid_arc.middle_y
+
+            # Adjust text anchor and rotation for better readability
+            transform = ""
+            if direction == "angled":
+                mid_angle = segment.relative_angle(0.5)
+                rotation_angle = self.get_text_rotation(mid_angle)
+                transform = f"rotate({rotation_angle}, {text_x}, {text_y})"
+
+            # Add text using the add_text method
+            self.add_text(
+                x=text_x,
+                y=text_y,
+                text=text,
+                fill=color,
+                font_weight="normal",
+                indent_level=indent_level,
+                transform=transform,
+                centered=True,
+            )
+
+        elif direction == "curved":
+            text_obj = Text(text, self.config)
+
+            for i, line in enumerate(text_obj.lines):
+                radial_offset = 1 - ((i + 1) / (text_obj.line_count + 1))
+                # Create a path for the text to follow
+                path_id = f"path{segment.start_angle}-{segment.end_angle}-{i}"
+                path_d = self.get_donut_path(
+                    segment, middle_arc=True, radial_offset=radial_offset
+                )
+                self.add_element(
+                    f'<path id="{path_id}" d="{path_d}" fill="none" stroke="none" />'
+                )
+
+                text_tag = f"""<text class="{text_class}" fill="{color}" font-family="{self.config.font}" font-size="{self.config.font_size}">"""
+                self.add_element(text_tag, indent_level=indent_level)
+                text_path = f"""<textPath  xlink:href="#{path_id}" startOffset="50%" dominant-baseline="middle" text-anchor="middle">{html.escape(line)}</textPath>"""
+                self.add_element(text_path, indent_level=indent_level + 1)
+                self.add_element("</text>", indent_level=indent_level)
+        else:
+            raise ValueError(f"invalid direction {direction}")
+
     def get_java_script(self) -> str:
         """
         get the java script code for interactive behavior
@@ -497,7 +697,7 @@ class SVG:
     """
         return popup_script
 
-    def get_svg_markup(self, with_java_script: bool = True) -> str:
+    def get_svg_markup(self, with_java_script: bool = False) -> str:
         """
         Generate the complete SVG markup.
 
@@ -517,7 +717,8 @@ class SVG:
             f'xmlns:xlink="http://www.w3.org/1999/xlink" '
             f'width="{self.width}" height="{self.config.total_height}">\n'
         )
-        popup = """
+        popup = (
+            """
         <!-- Add a foreignObject for the popup -->
 <foreignObject id="dcm-svg-popup" class="popup" width="500" height="354" x="150" y="260" visibility="hidden">
     <body xmlns="http://www.w3.org/1999/xhtml">
@@ -529,7 +730,9 @@ class SVG:
     </body>
 </foreignObject>
 """
-
+            if self.config.with_popup
+            else ""
+        )
         styles = self.get_svg_style()
         body = "".join(self.elements)
         footer = "</svg>"
